@@ -1,17 +1,31 @@
+import json
+import os
 import re
-import constants
+import socket
+import time
 import itertools
-import mimetools
 import mimetypes
-from urllib import urlencode, quote
+import six
+from random import randint
+from collections import OrderedDict
+
+if six.PY2:
+    from urllib import urlencode, quote
+else:
+    from urllib.parse import urlencode, quote
 
 import apysigner
 
 from . import request
+from generic_request_signer import constants
+from generic_request_signer.convert_values_to_list import ConvertValuesToList
 
 
-def default_encoding(raw_data, *args):
-    return urlencode(raw_data, doseq=True)
+def default_encoding(raw_data, querystring=False):
+    encoded_url = urlencode(OrderedDict(sorted(raw_data.items())), doseq=True)
+    if six.PY2 or querystring:
+        return encoded_url
+    return encoded_url.encode()
 
 
 def json_encoding(raw_data, *args):
@@ -28,6 +42,7 @@ class SignedRequestFactory(object):
         self.files = files
         self.content_type_encodings = {
             'application/json': json_encoding,
+            'application/vnd.api+json': json_encoding,
         }
 
     @property
@@ -43,7 +58,7 @@ class SignedRequestFactory(object):
     def build_request_url(self, url, headers):
         url = self._build_client_url(url)
         if self.should_data_be_sent_on_querystring():
-            url += "&{0}".format(urlencode(self.raw_data, doseq=True))
+            url += "&{0}".format(default_encoding(self.raw_data, querystring=True))
         return self._build_signed_url(url, headers)
 
     def _build_signed_url(self, url, headers):
@@ -56,18 +71,28 @@ class SignedRequestFactory(object):
         match = re.search(r'(^.+://)([^?]+)(\?.+$)?', url)
         return match.group(1) + quote(match.group(2)) + (match.group(3) if match.group(3) is not None else '')
 
-    def _build_signature_dict_for_content_type(self, headers):
+    def _build_signature_dict_for_content_type(self, headers):  # noqa: C901
         content_type = headers.get("Content-Type")
-        if content_type and content_type == "application/json":
+        if content_type and content_type in ["application/json", "application/vnd.api+json"]:
             encoding_func = self.content_type_encodings.get(content_type, default_encoding)
             return encoding_func(self.raw_data)
-        return self.raw_data
+        if self.raw_data:
+            multi_dict = ConvertValuesToList()
+            if type(self.raw_data) == str:
+                multi_dict.update(json.loads(self.raw_data))
+            else:
+                multi_dict.update(self.raw_data)
+            return dict(multi_dict)
+        return {}
 
     def _get_data_payload(self, request_headers):
         if self.raw_data and not self.method_uses_querystring():
             content_type = request_headers.get("Content-Type")
             encoding_func = self.content_type_encodings.get(content_type, default_encoding)
-            return encoding_func(self.raw_data)
+            encoded_data = encoding_func(self.raw_data)
+            if not six.PY2 and isinstance(self.raw_data, str):
+                return encoded_data.encode()
+            return encoded_data
 
     def should_data_be_sent_on_querystring(self):
         return self.method_uses_querystring() and self.raw_data
@@ -86,7 +111,8 @@ class MultipartSignedRequestFactory(SignedRequestFactory):
 
     def __init__(self, *args, **kwargs):
         super(MultipartSignedRequestFactory, self).__init__(*args, **kwargs)
-        self.boundary = mimetools.choose_boundary()
+        self.boundary_prefix = None
+        self.boundary = self.choose_boundary()
         self.part_boundary = "--" + self.boundary
 
     def create_request(self, url, *args, **request_kwargs):
@@ -97,8 +123,8 @@ class MultipartSignedRequestFactory(SignedRequestFactory):
 
     def _build_request(self, body, url):
         new_request = request.Request(self.http_method, url, None)
-        new_request.add_data(body)
-        new_request.add_header('Content-type', 'multipart/form-data; boundary=%s' % self.boundary)
+        new_request.data = body
+        new_request.add_header('Content-type', 'multipart/form-data; boundary={}'.format(self.boundary).encode())
         return new_request
 
     def get_multipart_body(self, data):
@@ -109,21 +135,43 @@ class MultipartSignedRequestFactory(SignedRequestFactory):
 
     def get_multipart_fields(self, data):
         for name, value in data.items():
-            yield [self.part_boundary, self.FIELD.format(name), '', str(value)]
+            yield [self.part_boundary.encode(), self.FIELD.format(name).encode(), ''.encode(), str(value).encode()]
 
-    def get_multipart_files(self):
+    def get_multipart_files(self):  # noqa: C901
         for input_file in self.input_files:
             for field_name, (filename, body) in input_file.items():
+                read_body = body.read()
+                if not isinstance(read_body, six.binary_type):
+                    read_body = read_body.encode()
                 yield [
-                    self.part_boundary, self.FILE.format(field_name, filename),
-                    self.get_content_type(filename), '', body.read()
+                    self.part_boundary.encode(),
+                    self.FILE.format(field_name, filename).encode(),
+                    self.get_content_type(filename), ''.encode(), read_body
                 ]
 
     def get_content_type(self, filename):
-        return 'Content-Type: {}'.format(mimetypes.guess_type(filename)[0] or 'application/octet-stream')
+        return 'Content-Type: {}'.format(mimetypes.guess_type(filename)[0] or 'application/octet-stream').encode()
 
     def flatten_multipart_body(self, parts):
         flattened = list(itertools.chain(*parts))
-        flattened.append(self.part_boundary + '--')
-        flattened.append('')
-        return '\r\n'.join(flattened)
+        boundary_part = self.part_boundary + '--'
+        flattened.append(boundary_part.encode())
+        flattened.append(''.encode())
+        return '\r\n'.encode().join(flattened)
+
+    def choose_boundary(self):  # noqa: C901
+        if not self.boundary_prefix:
+            try:
+                hostid = socket.gethostbyname(socket.gethostname())
+            except socket.gaierror:
+                hostid = '127.0.0.1'
+            try:
+                uid = repr(os.getuid())
+            except AttributeError:
+                uid = '1'
+            try:
+                pid = repr(os.getpid())
+            except AttributeError:
+                pid = '1'
+            self.boundary_prefix = hostid + '.' + uid + '.' + pid
+        return '{}.{}.{}'.format(self.boundary_prefix, time.time(), randint(1000, 10000))
