@@ -1,3 +1,4 @@
+import six
 import json
 
 
@@ -19,8 +20,10 @@ class Response(object):
     @property
     def json(self):
         response_content = self.read()
-        if response_content == '':
+        if response_content == ''.encode():
             return {}
+        if six.PY3 and isinstance(response_content, bytes):
+            response_content = bytes.decode(response_content, 'utf-8')
         return json.loads(response_content)
 
     @property
